
const path = require('path');
const fs = require('fs');

const pathMyRootNodeModules = path.resolve(__dirname, '../node_modules');
const pathWorkspaceNodeModules = path.resolve(process.cwd(), './node_modules');

exports.getLoaderAbsolutePath = function getLoaderAbsolutePath(loaderName) {
    let loaderPath = path.join(pathMyRootNodeModules, loaderName);
    if (fs.existsSync(loaderPath)) {
        return loaderPath;
    }

    loaderPath = path.join(pathWorkspaceNodeModules, loaderName);
    if (fs.existsSync(loaderPath)) {
        return loaderPath;
    }

    return loaderName;
};

function isObject(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
  
/**
 * Deep merge two objects.
 * @param target
 * @param ...sources
 */
exports.mergeDeep = function mergeDeep(target, ...sources) {
    if (!sources.length) return target;
    const source = sources.shift();

    if (isObject(target) && isObject(source)) {
        for (const key in source) {
            if (isObject(source[key])) {
                if (!target[key]) Object.assign(target, { [key]: {} });
                mergeDeep(target[key], source[key]);
            } else {
                Object.assign(target, { [key]: source[key] });
            }
        }
    }

    return mergeDeep(target, ...sources);
};

exports.getCliArgv = function (argvName) {
    const argvArr = process.argv;
    const argv = Array.prototype.find.call(argvArr, a => a.startsWith(`--${argvName}=`));
    
    return argv ? (argv.split('=')[1] || '') : '';
};

exports.cutPages = function (appConfig) {
    const pagesToCompile = exports.getCliArgv('pages').split(',');
    if (pagesToCompile.length) {
        const pages = [];
        pagesToCompile.forEach(pageName => {
            const page = appConfig.pages.find(p => p.path.endsWith(`/${pageName}`));

            if (page) {
                pages.push(page);
            }
        });

        if (pages.length) {
            appConfig.pages = pages;
        }
    }
};